@@ -61,6 +61,7 @@ void log_away_deinit(void);
 int irssi_gui;
 int irssi_init_finished;
 int reload_config;
+time_t client_start_time;
 
 static char *irssi_dir, *irssi_config_file;
 static GSList *dialog_type_queue, *dialog_text_queue;
@@ -211,10 +212,11 @@ static void sig_irssi_init_finished(void)
         irssi_init_finished = TRUE;
 }
 
-void core_init(int argc, char *argv[])
+void core_init(void)
 {
 	dialog_type_queue = NULL;
 	dialog_text_queue = NULL;
+	client_start_time = time(NULL);
 
 	modules_init();
 #ifndef WIN32
@@ -251,7 +253,7 @@ void core_init(int argc, char *argv[])
 	chat_commands_init();
 
 	settings_add_str("misc", "ignore_signals", "");
-	settings_add_bool("misc", "override_coredump_limit", TRUE);
+	settings_add_bool("misc", "override_coredump_limit", FALSE);
 
 #ifdef HAVE_SYS_RESOURCE_H
 	getrlimit(RLIMIT_CORE, &orig_core_rlimit);
