@@ -97,6 +97,24 @@ int strlen_big5(const unsigned char *str)
 	return len;
 }
 
+void unichars_to_big5_with_pos(const unichar *str, int spos, char *out, int *opos)
+{
+	const unichar *sstart = str;
+	char *ostart = out;
+
+	*opos = 0;
+	while(*str != '\0')
+	{
+		if(*str > 0xff)
+			*out ++ = (*str >> 8) & 0xff;
+		*out ++ = *str & 0xff;
+		str ++;
+		if(str - sstart == spos)
+			*opos = out - ostart;
+	}
+	*out = '\0';
+}
+
 void big5_to_unichars(const char *str, unichar *out)
 {
 	const unsigned char *p = (const unsigned char *) str;
@@ -214,22 +232,24 @@ static void gui_entry_draw_from(GUI_ENTRY_REC *entry, int pos)
 
 		if (entry->hidden)
                         term_addch(root_window, ' ');
-		else if (*p >= 32 && (entry->utf8 || (*p & 127) >= 32))
+		else if (*p >= 32 && (*p < 0x7F || *p > 0x9F))
 			term_add_unichar(root_window, *p);
 		else {
 			term_set_color(root_window, ATTR_RESET|ATTR_REVERSE);
-			term_addch(root_window, *p+'A'-1);
+			term_addch(root_window, (*p & 127)+'A'-1);
 			term_set_color(root_window, ATTR_RESET);
 		}
 	}
 
         /* clear the rest of the input line */
-        if (end_xpos == term_width)
-		term_clrtoeol(root_window);
-	else {
-		while (xpos < end_xpos) {
-                        term_addch(root_window, ' ');
-                        xpos++;
+	if (xpos < end_xpos) {
+		if (end_xpos == term_width)
+			term_clrtoeol(root_window);
+		else {
+			while (xpos < end_xpos) {
+				term_addch(root_window, ' ');
+				xpos++;
+			}
 		}
 	}
 }
@@ -371,6 +391,29 @@ char *gui_entry_get_text(GUI_ENTRY_REC *entry)
 	return buf;
 }
 
+char *gui_entry_get_text_and_pos(GUI_ENTRY_REC *entry, int *pos)
+{
+	char *buf;
+        int i;
+
+	g_return_val_if_fail(entry != NULL, NULL);
+
+	buf = g_malloc(entry->text_len*6 + 1);
+	if (entry->utf8)
+		utf16_to_utf8_with_pos(entry->text, entry->pos, buf, pos);
+	else {
+		if(term_type==TERM_TYPE_BIG5)
+			unichars_to_big5_with_pos(entry->text, entry->pos, buf, pos);
+		else
+		{
+			for (i = 0; i <= entry->text_len; i++)
+				buf[i] = entry->text[i];
+			*pos = entry->pos;
+		}
+	}
+	return buf;
+}
+
 void gui_entry_insert_text(GUI_ENTRY_REC *entry, const char *str)
 {
         unichar chr;
@@ -466,8 +509,18 @@ void gui_entry_erase_to(GUI_ENTRY_REC *entry, int pos, int update_cutbuffer)
 	gui_entry_erase(entry, size, update_cutbuffer);
 }
 
+static size_t cell_width(unichar *buf, int len)
+{
+	unichar *str = buf;
+
+	while (len-- && utf8_width(*str--) == 0);
+	return buf - str;
+}
+
 void gui_entry_erase(GUI_ENTRY_REC *entry, int size, int update_cutbuffer)
 {
+	size_t w = 0;
+
         g_return_if_fail(entry != NULL);
 
 	if (entry->pos < size)
@@ -491,13 +544,16 @@ void gui_entry_erase(GUI_ENTRY_REC *entry, int size, int update_cutbuffer)
 		return;
 	}
 
+	if (entry->utf8)
+		w = cell_width(entry->text + entry->pos - size, entry->pos - size + 1)-1;
+
 	g_memmove(entry->text + entry->pos - size, entry->text + entry->pos,
 		  (entry->text_len-entry->pos+1) * sizeof(unichar));
 
 	entry->pos -= size;
         entry->text_len -= size;
 
-	gui_entry_redraw_from(entry, entry->pos);
+	gui_entry_redraw_from(entry, entry->pos-w);
 	gui_entry_fix_cursor(entry);
 	gui_entry_draw(entry);
 }
