@@ -84,11 +84,11 @@ int quitting;
 static const char *firsttimer_text =
 	"Looks like this is the first time you've run irssi.\n"
 	"This is just a reminder that you really should go read\n"
-	"startup-HOWTO if you haven't already. Irssi's default\n"
-	"settings aren't probably what you've been used to, and you\n"
-	"shouldn't judge the whole client as crap based on them.\n\n"
-	"You can find startup-HOWTO and more irssi beginner info at\n"
-	"http://irssi.org/beginner/";
+	"startup-HOWTO if you haven't already. You can find it\n"
+	"and more irssi beginner info at http://irssi.org/help/\n"
+	"\n"
+	"For the truly impatient people who don't like any automatic\n"
+	"window creation or closing, just type: /MANUAL-WINDOWS";
 static int display_firsttimer = FALSE;
 
 
@@ -269,7 +269,7 @@ static void check_oldcrap(void)
 		return;
 	}
 
-	printf("\nYou seem to have old default.theme in "IRSSI_DIR_SHORT"/ directory.\n");
+	printf("\nYou seem to have an old default.theme in %s/ directory.\n", get_irssi_dir());
         printf("Themeing system has changed a bit since last irssi release,\n");
         printf("you should either delete your old default.theme or manually\n");
         printf("merge it with the new default.theme.\n\n");
@@ -309,6 +309,22 @@ static void winsock_init(void)
 }
 #endif
 
+#ifdef USE_GC
+#ifdef HAVE_GC_H
+#  include <gc.h>
+#else
+#  include <gc/gc.h>
+#endif
+
+GMemVTable gc_mem_table = {
+	GC_malloc,
+	GC_realloc,
+	GC_free,
+
+	NULL, NULL, NULL
+};
+#endif
+
 int main(int argc, char **argv)
 {
 	static struct poptOption options[] = {
@@ -316,6 +332,12 @@ int main(int argc, char **argv)
 		{ NULL, '\0', 0, NULL }
 	};
 
+#ifdef USE_GC
+	g_mem_set_vtable(&gc_mem_table);
+#endif
+
+	srand(time(NULL));
+
 	dummy = FALSE;
 	quitting = FALSE;
 	core_init_paths(argc, argv);
@@ -328,19 +350,17 @@ int main(int argc, char **argv)
 #ifdef HAVE_SOCKS
 	SOCKSinit(argv[0]);
 #endif
-#ifdef ENABLE_NLS
-	/* initialize the i18n stuff */
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
-#endif
 
 	/* setlocale() must be called at the beginning before any calls that
 	   affect it, especially regexps seem to break if they're generated
-	   before t his call.
+	   before this call.
 
 	   locales aren't actually used for anything else than autodetection
-	   of UTF-8 currently.. */
-	setlocale(LC_CTYPE, "");
+	   of UTF-8 currently..  
+
+	   furthermore to get the users's charset with g_get_charset() properly 
+	   you have to call setlocale(LC_ALL, "") */
+	setlocale(LC_ALL, "");
 
 	textui_init();
 	args_register(options);
@@ -358,7 +378,10 @@ int main(int argc, char **argv)
 	/* Does the same as g_main_run(main_loop), except we
 	   can call our dirty-checker after each iteration */
 	while (!quitting) {
-                if (!dummy) term_refresh_freeze();
+#ifdef USE_GC
+		GC_collect_a_little();
+#endif
+		if (!dummy) term_refresh_freeze();
 		g_main_iteration(TRUE);
                 if (!dummy) term_refresh_thaw();
 
