@@ -1,7 +1,7 @@
 /*
  irc-expandos.c : irssi
 
-    Copyright (C) 2000 Timo Sirainen
+    Copyright (C) 2000-2002 Timo Sirainen
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "settings.h"
 
 #include "irc-servers.h"
-#include "channels.h"
+#include "irc-channels.h"
 #include "nicklist.h"
 
 static char *last_join;
@@ -66,7 +66,7 @@ static char *expando_userhost(SERVER_REC *server, void *item, int *free_ret)
 
 	/* haven't received userhost reply yet. guess something */
 	*free_ret = TRUE;
-	if (server == NULL)
+	if (ircserver == NULL)
 		username = settings_get_str("user_name");
 	else
 		username = ircserver->connrec->username;
@@ -82,10 +82,19 @@ static char *expando_usermode(SERVER_REC *server, void *item, int *free_ret)
 	return IS_IRC_SERVER(server) ? IRC_SERVER(server)->usermode : "";
 }
 
-/* expands to your usermode on channel, op '@', halfop '%', "+" voice */
+/* expands to your usermode on channel, op '@', halfop '%', "+" voice or other */
 static char *expando_cumode(SERVER_REC *server, void *item, int *free_ret)
 {
-	if (IS_CHANNEL(item) && CHANNEL(item)->ownnick) {
+	if (IS_IRC_CHANNEL(item) && CHANNEL(item)->ownnick) {
+		char other = NICK(CHANNEL(item)->ownnick)->other;
+		if (other != '\0') {
+			char *cumode = g_malloc(2);
+			cumode[0] = other;
+			cumode[1] = '\0';
+			*free_ret = TRUE;
+			return cumode;
+		}
+
 		return NICK(CHANNEL(item)->ownnick)->op ? "@" :
 		       NICK(CHANNEL(item)->ownnick)->halfop ? "%" :
 		       NICK(CHANNEL(item)->ownnick)->voice ? "+" : "";
@@ -98,7 +107,10 @@ static char *expando_cumode(SERVER_REC *server, void *item, int *free_ret)
 static char *expando_cumode_space(SERVER_REC *server, void *item, int *free_ret)
 {
 	char *ret;
-	
+
+	if (!IS_IRC_SERVER(server))
+                return "";
+
 	ret = expando_cumode(server, item, free_ret);
 	return *ret == '\0' ? " " : ret;
 }
