@@ -31,11 +31,11 @@ static void ban_free(GSList **list, BAN_REC *rec)
 	g_return_if_fail(list != NULL);
 	g_return_if_fail(rec != NULL);
 
+	*list = g_slist_remove(*list, rec);
+
 	g_free(rec->ban);
 	g_free_not_null(rec->setby);
 	g_free(rec);
-
-	*list = g_slist_remove(*list, rec);
 }
 
 void banlist_free(GSList *banlist)
@@ -88,7 +88,7 @@ BAN_REC *banlist_add(IRC_CHANNEL_REC *channel, const char *ban,
 	return rec;
 }
 
-void banlist_remove(IRC_CHANNEL_REC *channel, const char *ban)
+void banlist_remove(IRC_CHANNEL_REC *channel, const char *ban, const char *nick)
 {
 	BAN_REC *rec;
 
@@ -97,7 +97,7 @@ void banlist_remove(IRC_CHANNEL_REC *channel, const char *ban)
 
 	rec = banlist_find(channel->banlist, ban);
 	if (rec != NULL) {
-		signal_emit("ban remove", 2, channel, rec);
+		signal_emit("ban remove", 3, channel, rec, nick);
 		ban_free(&channel->banlist, rec);
 	}
 }
